/*@***************************************************************************
@
@       PROJECT         : XB: Sorter Control
@
@       FILE            : MOVECELL.C
@
@       VERSION         : 4.1
@
@       TYPE            : 'C' TASK FILE
@
@
@       DESCRIPTION     : Send movement/cancel movement commands to a cell
@
@       COMPILER        : ic386
@
@       OPERATING SYSTEM: iRMX for Windows
@
@       NOTES           : 28/06/96 - a) catalog task/job,
@                                    b) use of the ESCAPE video sequences
@                                    c) set user CTRL-C to flush CTRL_C
@                                       for use in Maintenance VDT function
@
@                         17/07/97 - New "cell_valid" routine the check
@                         if the number of requested cell to move exist
@													12/12/97 - Avoid to run the program on the IC
@													20/01/97 - Bug correction: modification for "max_cell" instead
@                         of "gn_cell" and for the "cell_valid" routine
@
@***************************************************************************/


#define MAIN
#define TASK

#include "movecell.h"

/*@.....................................................................
@
@ Global variables and definex
@
@.....................................................................*/

int 	direction=CW_DIRECTION;
int 	profile=45;
int 	delay_move=100;
int 	delay_cancel=100;
int 	delay_ms;
int 	cell_from=1;
int 	cell_to;
int 	cell_delay=1000;	/* DAN 27/09/99 per evitare di bruciare i fusibili */
int 	times_to_move=10;
int 	max_cell;
char 	*des_direction[]={"CW_DIRECTION","CCW_DIRECTION"};
char 	mask1[81];
char 	mask2[81];
char 	mask3[81];

#define TITLE "MOVECELL: DCS Maintenance Program"
#define SUBTITLE 	"Cell test"

/*@.....................................................................
@
@ Job task prototypes
@
@.....................................................................*/

short cell_valid(int value, short prin);
int 	input_data(int min,int max,int row,char *string);
void 	set_move(int n_cell);
void 	set_delete(int n_cell);
void 	clearscreen(void);
void 	GOTOXY(int y,int x);
void 	CLRTOEND(void);
void 	CLRTOENDOFLINE(void);

/*@..........................................................................
@
@  Begin of the initial task of the job
@
@...........................................................................*/

void main(void)
{
DEFDBLOCVAR(cell);
char 		buffer[MAX_BUFDIM];
int 		fase=0;
int 		value,i;
S_INIT 	s_INIT;
TOKEN  	src;
UCHAR 	priority;
char		*s;

	set_exception_inline();

/*@..........................................................................
@
@   Get MBX_D_INIT mailbox token and wait on the mailbox
@
@...........................................................................*/

	GET_TOKEN_GLOBAL(mbx_init);
	mbx_data_wait_forever(s_id_token_mbx_init.id,(POINTER)buffer);
	memcpy((POINTER)&s_INIT,get_internal_message_text(buffer),sizeof(S_INIT));
	src=get_internal_message_src(buffer);
/*@..........................................................................
@
@   Get DB TOKEN and attach the static DB and all the TABLE
@
@...........................................................................*/

	gdb_segment=s_INIT.dbase;

#include "attach.h"
/*@..........................................................................
@
@   Catalog task
@
@...........................................................................*/

	catalog_task_local("TSK_MOVECELL",get_task_id((POINTER)0));

/*@..........................................................................
@
@   Get the token or create global resources
@
@...........................................................................*/

	s_id_token_mbx_ccmgm.id = create_FIFO_D_mailbox(s_id_token_mbx_ccmgm.name);
	s_id_token_mbx_srldmes.id = create_FIFO_D_mailbox(s_id_token_mbx_srldmes.name);
	s_id_token_mbx_logerr.id = create_FIFO_D_mailbox(s_id_token_mbx_logerr.name);

/*@..........................................................................
@
@   Get the TOKEN of the user CTRL-C semaphore and install it
@
@...........................................................................*/
	GET_TOKEN_LOCAL(sem_ctrl_c,semaphore);
	set_ctrl_c_mgm(s_id_token_sem_ctrl_c.id);

/*@..........................................................................
@
@   Enable the new job sending a message to the MBX_D_INIT mailbox
@
@...........................................................................*/

	RESEND_MES(INIT,MS_NONE,s_id_token_mbx_init.id,src);

	memset(mask1,'*',sizeof(mask1));
	mask1[80]=0;
	memset(mask2,' ',sizeof(mask2));
	mask2[0]='*';
	s=strchr(xb_version,'=');
	if (s)
		sprintf(buffer,"%s - %s",TITLE,s+1);
	else
		sprintf(buffer,"%s",TITLE);
	memcpy(&mask2[(sizeof(mask2)-strlen(buffer))/2],buffer,strlen(buffer));
	mask2[79]='*';
	mask2[80]=0;
	memset(mask3,' ',sizeof(mask3));
	memcpy(&mask3[(sizeof(mask3)-strlen(SUBTITLE))/2],SUBTITLE,strlen(SUBTITLE));
	mask3[80]=0;

	/* 12/12/97 - Avoid to run the program on the IC */
	if (!gn_cell)
		{
		clearscreen();
		printf("Program not executable on IC\n");
		sleep_task(2000/TICK);
		exit(0);
		}

	max_cell=gps_cell[gn_cell-1].id;
	cell_to=max_cell;

	FOREVER
		{
		switch(fase)
			{
			case 0:
				clearscreen();
				printf("0 - This menu\n");
				printf("1 - Cell to move\n");
				printf("2 - Cell to move and delete before move\n");
				printf("3 - Profile (actual=%d)\n",profile);
				printf("4 - Delay in ms before motion (actual=%d)\n",delay_move);
				printf("5 - Delay in ms before delete motion (actual=%d)\n",delay_cancel);
				printf("6 - Direction (actual=%s)\n",des_direction[direction]);
				printf("7 - Move from the cell %d to the cell %d with delay %d\n",cell_from,cell_to,cell_delay);
				printf("8 - Move the cell: CW and CCW\n");
				printf("9 - Move the cell CW and CCW for n times\n");
				printf("10 - Exit\n");
				value=input_data(0,10,16,"Select");
				if (value==10)
					exit(0);
				else
					{
					fase=value;
					}
				break;



			case 1:
				/* CELL to MOVE */
				GOTOXY(4,1);
				CLRTOEND();
				printf("\n0 = Main menu\n");
				sprintf(buffer,"Cell to move (from 1 to %d)",max_cell);
				value=input_data(1,max_cell,6,buffer);
				if (!cell_valid(value,4))
					fase=0;
				else
					set_move(value);
				break;

			case 2:
				/* CELL to MOVE and DELETE before MOVE */
				GOTOXY(4,1);
				CLRTOEND();
				printf("\n0 = Main menu\n");
				sprintf(buffer,"Cell to move (from 1 to %d)",max_cell);
				value=input_data(1,max_cell,6,buffer);
				if (!cell_valid(value,4))
					fase=0;
				else
				 {
					priority=get_priority();
					change_priority(117);
					set_move(value);
					delay_ms=delay_move-delay_cancel;
					if (delay_ms<0)
						delay_ms=0;
					sleep_task(delay_ms/TICK);
					set_delete(value);
					change_priority(priority);
					}
				break;

			case 3:
				/* PROFILE choice */
				GOTOXY(4,1);
				CLRTOEND();
				printf("\n0 = Main menu\n");
				sprintf(buffer,"Profile (from 1 to 45 - actual=%d)",profile);
				value=input_data(1,45,6,buffer);
				if (value==0)
					fase=0;
				else
					profile=value;
				break;

			case 4:
				/* DELAY BEFORE MOTION */
				GOTOXY(4,1);
				CLRTOEND();
				printf("\n0 = Main menu\n");
				sprintf(buffer,"Delay in ms before motion (from 500 to 5000 - actual=%d)",delay_move);
				value=input_data(500,5000,6,buffer);
				if (value==0)
					fase=0;
				else
					delay_move=value;
				break;

			case 5:
				/* DELAY BEFORE DELETE MOTION */
				GOTOXY(4,1);
				CLRTOEND();
				printf("\n0 = Main menu\n");
				sprintf(buffer,"Delay in ms before delete motion (from 1 to 1000 - actual=%d)",delay_cancel);
				value=input_data(1,1000,6,buffer);
				if (value==0)
					fase=0;
				else
					delay_cancel=value;
				break;


			case 6:
				/* DIRECTION choice */
				GOTOXY(4,1);
				CLRTOEND();
				printf("\n0 = Main menu\n");
				sprintf(buffer,"Direction (1=CW 2=CCW - actual=%s)",des_direction[direction]);
				value=input_data(1,2,6,buffer);
				if (value==0)
					fase=0;
				else
					direction=value-1;
				break;

			case 7:
				/* GROUP of CELLs to MOVE */
				GOTOXY(4,1);
				CLRTOEND();
				printf("\n0 = Main menu\n");
				printf("Move from the cell %d to the cell %d with delay %d\n",cell_from,max_cell,cell_delay);
				value=input_data(1,max_cell,7,"Cell from");
				if (!cell_valid(value,4))
					{
					fase=0;
					break;
					}
				else
					cell_from=value;
				value=input_data(cell_from,max_cell,8,"Cell to");
				if (!cell_valid(value,4))
					{
					fase=0;
					break;
					}
				else
					cell_to=value;
				/* DAN 27/09/99 per evitare di bruciare i fusibili */
				value=input_data(1000,2000,9,"Delay between motions");
				if (value==0)
					{
					fase=0;
					break;
					}
				else
					cell_delay=value;


				FOREVER
					{
					priority=get_priority();
					change_priority(117);
					GOTOXY(10,1);
					CLRTOEND();
					for (value=cell_from;value<=cell_to;value++)
						{
						if (cell_valid(value,0))
							{
							GOTOXY(10,1);
							CLRTOEND();
							printf("Move cell %d\n",value);
							set_move(value);
							sleep_task(cell_delay/TICK);
							}
						else
							{
							sleep_task(0);
							}
						}
					change_priority(priority);
					GOTOXY(11,1);
					CLRTOEND();
					value=input_data(0,1,11,"0=Main menu - 1=continue");
					if (value==0)
						{
						fase=0;
						break;
						}
					}
				break;

			case 8:
				/* CELL to MOVE : CW and CCW*/
				GOTOXY(4,1);
				CLRTOEND();
				printf("\n0 = Main menu\n");
				sprintf(buffer,"Cell to move: CW and CCW (from 1 to %d)",max_cell);
				value=input_data(1,max_cell,6,buffer);

				if (value==0)
					fase=0;
				else
					{
					direction=0;
					delay_ms=50;
					set_move(value);
					GOTOXY(7,1);
					CLRTOEND();
					printf("Press CR to move the cell in CCW direction..\n");
					gets(buffer);
					direction=1;
					delay_ms=50;
					set_move(value);
					}
				break;

			case 9:
				/* CELL to MOVE n TIMES */
				GOTOXY(4,1);
				CLRTOEND();
				printf("\n0 = Main menu\n");
				sprintf(buffer,"Cell to move CW and CCW n times (from 1 to %d)",
						 max_cell);
				value=input_data(1,max_cell,6,buffer);

				if (value==0)
					fase=0;
				else
					{
					int cell_to_move,old_direction;

					cell_to_move=value;
					sprintf(buffer,"Number of motions (from 1 to 100) - Actual=%d",
									 times_to_move);
					value=input_data(0,100,7,buffer);
					if (value)
						times_to_move=value;
					old_direction=direction;
					for (direction=0;direction<=1;direction++)
						{
						for (value=1;value<=times_to_move;value++)
							{
							GOTOXY(8,1);
							printf("Motion #%d for the cell %d (direction=%s)\n",
											value,
											cell_to_move,
											des_direction[direction]);
							set_move(cell_to_move);
							if (profile<40)
								sleep_task(800/TICK);
							else
								sleep_task(1500/TICK);
							}
						}
					direction=old_direction;
					}
				break;
			}
	}

}
/*@..........................................................................
@  end  of MOVECELL task/job
@***************************************************************************/



/*@..........................................................................
@
@  move the cell
@
@...........................................................................*/
void set_move(int n_cell)
{
	if (n_cell)
		celcmd_reservation(n_cell,direction,delay_move,profile,(USHORT)DEV_CHUTE);
}

/*@..........................................................................
@
@  delete the cell moving
@
@...........................................................................*/
void set_delete(int n_cell)
{
	if (n_cell)
		celcmd_cancel(n_cell);
}

/*@..........................................................................
@
@  get the operator input from keyboard and set the input maiuscole
@
@...........................................................................*/


int input_data(int min,int max,int row,char *string)
{
char 		buffer[128];
int 		value;
USHORT	unit;

	GOTOXY(row,1);
	CLRTOEND();
	while(1)
		{
		GOTOXY(row,1);
		printf ("%s --> ",string);
		gets(buffer);
		CLRTOEND();
		/* flush the CTRL-C semaphore */
		unit=test_ctrl_c(s_id_token_sem_ctrl_c.id);
		if (unit)
			{
			*buffer=0;
			}
		value = atoi(buffer);
		if (!value)
			return(0);
		if (value<min || value > max)
			{
			printf("ERROR: wrong selection (valid from %d to %d)\n",min,max);
			}
		else
			break;
		}
	return(value);
}


/*@..........................................................................
@
@  Clear screen and dispay the program title
@
@...........................................................................*/

void clearscreen(void)
{
	printf("%c[2J",ESCKEY);
	printf("%s",mask1);
	printf("%s",mask2);
	printf("%s",mask1);
	printf("%s\n",mask3);
}

/*@..........................................................................
@
@  Set cursor to to row(=y) - column (=X)
@
@...........................................................................*/
void GOTOXY(int y,int x)
{
	printf("%c[%d;%dH",ESCKEY,y,x);
}
/*@..........................................................................
@
@  Clear screen from cursor to end of screen
@
@...........................................................................*/
void CLRTOEND(void)
{
	printf("%c[0J",ESCKEY);
}
/*@..........................................................................
@
@  Clear line from cursor to end of line
@
@...........................................................................*/
void CLRTOENDOFLINE(void)
{
	printf("%c[0K",ESCKEY);
}


/*@..........................................................................
@
@  Check if the number of cell exist
@
@...........................................................................*/
short cell_valid(int value, short prin)
{
	DEFDBLOCVAR(cell);

	TEST_CELL_ID_AND_GET_CELL(value);
	if (!CELL)
		{
		if(!prin)
			{
			GOTOXY(prin,1);
			CLRTOEND();
			printf("ERROR: the cell %d don't exist\n",value);
			}
		return 0;
		}

	return 1;

}