/*@***************************************************************************
@
@       PROJECT         : XB: Sorter Control
@
@       FILE            : UTILSC.C
@
@       VERSION         : XB5.0 
@
@       TYPE            : LIBRARY
@
@
@       DESCRIPTION     : General SC utility routines
@
@       COMPILER        : ic386
@
@       OPERATING SYSTEM: iRMX for Windows
@
@       NOTES           : 
@       11/04/95: new routine "compute_average" to compute
@       the average velocity of the XB
@       20/02/97 moved some routines in the files UTILCHUT.C,
@       UTILPARC.C, UTILCELL.C and UTILDEC.C
@       11/07/97 Set "recirc_reason" in the CHUTEREDREQ message
@       27/10/98 DAN - GENESIS: you have to consider the sorter level
@       level in set/reset cells errors in case of DECODER in failure
@       2002 Add QUAD management
@     	30/11/02 MIKE - VARIABLE cell pitch management
@
@***************************************************************************/

#include "xblib.h"

#include "scgen.h"
#include "scenc.h"
#include "scvel.h"
#include "cell.h"
#include "celcmd.h"
#include "decoder.h"


extern S_id_token s_id_token_mbx_ccmgm;
extern S_id_token s_id_token_mbx_srldmes;

/*@**************************************************************************
@
@ FUNCTION      : set_offl_error
@
@ DESCRIPTION   : when there is a offloading error increase the error
@                 counter for cell(if cell_error=1), decoder and converter
@
@ PROTOTYPE     : void set_offl_error(S_cell *ps_cell, USHORT error_type)
@
@ PARAMETERS    : ps_cell : POINTER to the data base of cell under
@                           test
@
@                 error_type:  bit 0=CELL_ERROR = set cell errors
@                              bit 1=DECODER_ERROR = set decoder errors
@                              bit 2=CONVERTER_ERROR = set converter errors
@ RETURN        : none
@
@ NOTES         :
@
@***************************************************************************/
void set_offl_error(S_cell *ps_cell, USHORT error_type)
{
DEFDBLOCVAR(cell);
DEFDBLOCVAR(decoder);
DEFDBLOCVAR(celcmd);
ULONG flag_double;


	CELL=ps_cell;
	if (TESTBIT(CELL->status,CELL_FAILURE))
		return;

	flag_double=FIRST_OF_DOUBLE_CELL;

	GET_DECODER(CELL);
	if (DECODER==(S_decoder *)0)
		return;
	GET_CONVERTER(CELL);
	if (CONVERTER==(S_celcmd *)0)
		return;
/*@**************************************************************************
@
@ If the error occured on the same cell and the error
@ for the cell must be tested:
@   increments the cell error counter
@ else if the last error occurred on the same decoder
@   increments the decoder error counter
@ else increments the converter error counter
@
@***************************************************************************/

	if (TESTBIT(error_type,CELL_ERROR))
		{
		if (CELL_HEADER->n_max_error)
			{
			if (++CELL->s_wk_cell.n_error == CELL_HEADER->n_max_error )
				{
				/* DAN 04/08/99 POST 53: deve considerare anche il nuovo bit di CELL_ITEM_FOUND */
				RESETBITEXCEPT(CELL->status,CELL_ITEM_FOUND);
        /* ASDA 28/01/2003 CM: deve considerare pure il bit di CELL_BUSY */
				if (TESTBIT(CELL->status,CELL_ITEM_FOUND))
					{
					SETBIT(CELL->status,CELL_BUSY);
					}
				/* SETBITEXCLUSIVE(CELL->status,CELL_FAILURE); */
				SETBIT(CELL->status,CELL_FAILURE);
				SETBIT(CELL->s_failure.logical,CELL_NO_MOVING);

				send_error(MS_ERROR_BEGIN,
							 DEV_CARRIER,
							 CELL->id,
							 CELL_NO_MOVING+9);
				CELL->s_wk_cell.n_error=0;
				}
			}
		}

	if (TESTBIT(error_type,DECODER_ERROR))
		{
		if (DECODER_HEADER->n_max_error)
			{
			if (++DECODER->s_wk_decoder.n_error == DECODER_HEADER->n_max_error)
				{
	/************************************************************************
	@
	@ Set the Decoder failure and send error to the LOGERR,
	@ set all the cell of Decoder in failure and send for every cell the
	@ error to the LOGERR,
	@ send a alarm to the SCMGM
	@
	************************************************************************/
				SETBIT(DECODER->s_failure.physical,DECODER_FAILURE);
				error_mgm(DEV_DECODER,DECODER->id,&DECODER->s_failure);

				{
				register int i;

        /* DAN 27/10/98 GENESIS: you have to consider the sorter level */
        TEST_CELL_ID_AND_GET_CELL(DECODER->s_cfg_decoder.n_first_cell);
        if (!CELL)
        	return;

				for (i = DECODER->s_cfg_decoder.n_first_cell;
					 i <= DECODER->s_cfg_decoder.n_last_cell;
					 i++,CELL++)
					{
					/* CELL=gps_cell-1+i; */
					/* DAN 04/08/99 POST 53: deve considerare anche il nuovo bit di CELL_ITEM_FOUND */
					RESETBITEXCEPT(CELL->status,CELL_ITEM_FOUND);
          /* ASDA 28/01/2003 CM: deve considerare pure il bit di CELL_BUSY */
					if (TESTBIT(CELL->status,CELL_ITEM_FOUND))
						{
						SETBIT(CELL->status,CELL_BUSY);
						}
					/* SETBITEXCLUSIVE(CELL->status,CELL_FAILURE); */
					SETBIT(CELL->status,CELL_FAILURE);
					SETBIT(CELL->s_failure.logical,CELL_FAILURE_DECODER);

					send_error(MS_ERROR_BEGIN,
								 DEV_CARRIER,
								 CELL->id,
								 CELL_FAILURE_DECODER+9);
					}
				}
				DECODER->s_wk_decoder.n_error=0;
				}
			}
		}

	if (TESTBIT(error_type,CONVERTER_ERROR))
		{
		if (CONVERTER_HEADER->n_max_error)
			{
			if (++CONVERTER->s_wk_celcmd.n_error == CONVERTER_HEADER->n_max_error)
				{
				SETBIT(CONVERTER->s_failure.physical,CONVERTER_FAILURE);
				error_mgm(DEV_CONVERTER,CONVERTER->id,&CONVERTER->s_failure);

				CONVERTER->s_wk_celcmd.n_error=0;
				}
			}
		}

/**********************************************************************************
@
@ If the parcel is on a DOUBLE cell:
@ if the error occured on the same cell
@ and the error for the cell must be tested:
@ increments the cell error counter
@ else if the encoder of the first cell of the DOUBLE cell is
@ different from the decoder of the second cell of the DOUBLE cell
@ and the last error occurred on the same decoder
@ increments the decoder error counter
@
@**********************************************************************/

	if (flag_double)
		{
		register USHORT n_decoder_first;

    CELL=ps_cell;
		GET_CELL_NEXT;
		n_decoder_first=DECODER->id;
		GET_DECODER(CELL);
		if (DECODER==(S_decoder *)0)
			return;

		/* It tests if the error occurs for the same cell */
		if (TESTBIT(error_type,CELL_ERROR))
			{
			if (CELL_HEADER->n_max_error)
				{
				if (++CELL->s_wk_cell.n_error == CELL_HEADER->n_max_error )
					{
					/* DAN 04/08/99 POST 53: deve considerare anche il nuovo bit di CELL_ITEM_FOUND */
					RESETBITEXCEPT(CELL->status,CELL_ITEM_FOUND);
	        /* ASDA 28/01/2003 CM: deve considerare pure il bit di CELL_BUSY */
					if (TESTBIT(CELL->status,CELL_ITEM_FOUND))
						{
						SETBIT(CELL->status,CELL_BUSY);
						}
					/* SETBITEXCLUSIVE(CELL->status,CELL_FAILURE); */
					SETBIT(CELL->status,CELL_FAILURE);
					SETBIT(CELL->s_failure.logical,CELL_NO_MOVING);

					send_error(MS_ERROR_BEGIN,
								 DEV_CARRIER,
								 CELL->id,
								 CELL_NO_MOVING+9);
					CELL->s_wk_cell.n_error=0;
					}
				}
			}

		if (TESTBIT(error_type,DECODER_ERROR))
			{
			if (n_decoder_first != DECODER->id)
				{
				if (++DECODER->s_wk_decoder.n_error == DECODER_HEADER->n_max_error)
					{
					register int i;
					/************************************************************************
					@
					@ Set the Decoder failure and send error to the LOGERR,
					@ set all the cell of Decoder in failure and send for every cell the
					@ error to the LOGERR,
					@ send a alarm to the SCMGM
					@
					************************************************************************/
					SETBIT(DECODER->s_failure.physical,DECODER_FAILURE);
					error_mgm(DEV_DECODER,DECODER->id,&DECODER->s_failure);

					/* DAN 27/10/98 GENESIS: you have to consider the sorter level */
          TEST_CELL_ID_AND_GET_CELL(DECODER->s_cfg_decoder.n_first_cell);
          if (!CELL)
            return;

					for (i = DECODER->s_cfg_decoder.n_first_cell;
						 i < DECODER->s_cfg_decoder.n_last_cell;
						 i++,CELL++)
						{
						/* CELL=gps_cell-1+i; */
						/* DAN 04/08/99 POST 53: deve considerare anche il nuovo bit di CELL_ITEM_FOUND */
						RESETBITEXCEPT(CELL->status,CELL_ITEM_FOUND);
     	      /* ASDA 28/01/2003 CM: deve considerare pure il bit di CELL_BUSY */
						if (TESTBIT(CELL->status,CELL_ITEM_FOUND))
							{
							SETBIT(CELL->status,CELL_BUSY);
							}
						/* SETBITEXCLUSIVE(CELL->status,CELL_FAILURE); */
						SETBIT(CELL->status,CELL_FAILURE);
						SETBIT(CELL->s_failure.logical,CELL_FAILURE_DECODER);


						send_error(MS_ERROR_BEGIN,
									 DEV_CARRIER,
									 CELL->id,
									 CELL_FAILURE_DECODER+9);
						}
					}
				}
			}
		}
}

/*@..........................................................................
@  end of the set_offl_error routine
@***************************************************************************/

/*@**************************************************************************
@
@ FUNCTION      : reset_offl_error
@
@ DESCRIPTION   : when there is a correct offloading, reset the error
@                 counter for cell(if cell_error=1), decoder and converter
@
@ PROTOTYPE     : void reset_offl_error(S_cell *ps_cell, USHORT cell_error)
@
@ PARAMETERS    : ps_cell : POINTER to the data base of cell under test
@                 cell_error : 0 = reset the error counters on decoder and
@                 converter
@                 1 = reset the error counter on cell, decoder and converter
@
@ RETURN        : none
@
@ NOTES         :
@
@***************************************************************************/
void reset_offl_error(S_cell *ps_cell, USHORT cell_error)
{
DEFDBLOCVAR(decoder);
DEFDBLOCVAR(celcmd);
DEFDBLOCVAR(cell);
ULONG flag_double;

	CELL=ps_cell;
	flag_double=FIRST_OF_DOUBLE_CELL;

	GET_DECODER(CELL);
	if (DECODER==(S_decoder *)0)
		return;

	GET_CONVERTER(CELL);
	if (CONVERTER==(S_celcmd *)0)
		return;

/*@**************************************************************************
@
@ Reset the error counter on cell (if cell_error=1), decoder and converter
@
@***************************************************************************/

	if (cell_error)
		CELL->s_wk_cell.n_error=0;

	DECODER->s_wk_decoder.n_error=0;
	CONVERTER->s_wk_celcmd.n_error=0;

/**********************************************************************************
@
@ If the parcel is on a DOUBLE cell:
@ if enabled, reset the cell counter error
@
@ If the DECODER number is different than the first cell, reset the error counter
@ on the new DECODER
@
@**********************************************************************/

	if (flag_double)
		{
		register USHORT n_decoder_first;

		GET_CELL_NEXT;
		n_decoder_first=DECODER->id;
		GET_DECODER(CELL);
		if (DECODER==(S_decoder *)0)
			return;


		/* Test if the error occurs for the same cell */
		if (cell_error)
			CELL->s_wk_cell.n_error=0;

		if (n_decoder_first != DECODER->id)
			{
			DECODER->s_wk_decoder.n_error=0;
			}
		}
}

/*@..........................................................................
@  end of the reset_offl_error routine
@***************************************************************************/


/*@**************************************************************************
@
@ FUNCTION      : celcmd_reservation
@
@ DESCRIPTION   : it sends the Cell Activation Command to the CCMGM task
@
@ PROTOTYPE     : void celcmd_reservation(USHORT n_cell,
@                 USHORT direction, long delay, USHORT profile.
@                 USHORT device);
@
@ PARAMETERS    : n_cell = cell number
@                 direction=belt movement direction (CW or CCW)
@                 delay=delay in ms
@                 profile=profile to use in belt movement
@                 device=type of device (TEST STATION,RECENTERING,CHUTE,ecc)
@
@ RETURN        : none
@
@ NOTES         : none
@
@***************************************************************************/
void celcmd_reservation(USHORT n_cell,USHORT direction,long delay,USHORT profile,USHORT device)
{
S_CELLACT s_CELLACT;
register long ep_time=XBVEL->s_wk_vel.ep_time;

	s_CELLACT.cell=n_cell;
	s_CELLACT.direction=direction;
	s_CELLACT.profile=profile;
	s_CELLACT.device=device;

	if (!delay)
		{
		s_CELLACT.delay_ep=0;
		s_CELLACT.delay_ms=0;
		}
	else
		{
		if (delay>1200)
			{
			s_CELLACT.delay_ep=(delay-1200+ep_time)/ep_time;
			s_CELLACT.delay_ms=delay-s_CELLACT.delay_ep*ep_time;
			}
		else
			{
			s_CELLACT.delay_ep=0;
			s_CELLACT.delay_ms=delay;
			}
		}

	SEND_MES(CELLACT,MS_CELLACT_SET,s_id_token_mbx_ccmgm.id);

}
/*@..........................................................................
@  end of the celcmd_reservation routine
@***************************************************************************/

/*@**************************************************************************
@
@ FUNCTION      : celcmd_cancel
@
@ DESCRIPTION   : it cancels a Cell Activation Command from the CCMGM task
@
@ PROTOTYPE     : void celcmd_cancel(USHORT n_cell)
@
@ PARAMETERS    : n_cell = cell number
@
@ RETURN        : none
@
@ NOTES         : none
@
@***************************************************************************/
void celcmd_cancel(USHORT n_cell)
{

S_CELLACT s_CELLACT;


	s_CELLACT.cell=n_cell;
	s_CELLACT.direction=
	s_CELLACT.delay_ms=
	s_CELLACT.delay_ep=
	s_CELLACT.profile=
	s_CELLACT.device=0;

	SEND_MES(CELLACT,MS_CELLACT_CANCEL,s_id_token_mbx_ccmgm.id);

}
/*@..........................................................................
@  end of the celcmd_cancel routine
@***************************************************************************/
/*@**************************************************************************
@
@ FUNCTION      : celcmd_reset_all
@
@ DESCRIPTION   : it resets all the Cell Activation Command from the CCMGM task
@
@ PROTOTYPE     : void celcmd_reset_all()
@
@ PARAMETERS    : none
@
@ RETURN        : none
@
@ NOTES         : none
@
@***************************************************************************/
void celcmd_reset_all(void)
{

S_CELLACT s_CELLACT;


	s_CELLACT.cell=1;
	s_CELLACT.direction=
	s_CELLACT.delay_ms=
	s_CELLACT.delay_ep=
	s_CELLACT.profile=
	s_CELLACT.device=0;

	SEND_MES(CELLACT,MS_CELLACT_RESET,s_id_token_mbx_ccmgm.id);

}
/*@..........................................................................
@  end of the celcmd_reset_all routine
@***************************************************************************/
/*@**************************************************************************
@
@ FUNCTION      : send_redestination_message
@
@ DESCRIPTION   : it sends a redestination message to the SRLDMES task
@
@ PROTOTYPE     : void send_redestination_message(S_cell *ps_cell_ele)
@
@ PARAMETERS    : ps_cell_cell: pointer to the data of cell to offload
@
@ RETURN        : none
@
@ NOTES         : 11/07/97 - Set "recirc_reason" in the CHUTEREDREQ message 
@
@***************************************************************************/
void send_redestination_message(S_cell *ps_cell_ele)
{
S_CHUTEREDREQ s_CHUTEREDREQ;
USHORT  i;

	s_CHUTEREDREQ.cell=CELL->id;
	s_CHUTEREDREQ.chute=CELL->s_parcres.s_parcel.chute;
	s_CHUTEREDREQ.parcid=CELL->s_parcres.s_parcel.parcid;
	s_CHUTEREDREQ.recirc_reason=(USHORT)MAX_RECIRC_REASON;
	for (i=0;i<(USHORT)MAX_RECIRC_REASON;i++)
		{
		if (TESTBIT(CELL->s_wk_cell.recirc_reason,i))
			{
			s_CHUTEREDREQ.recirc_reason=i+1;
			break;
			}
		}

	memcpy(s_CHUTEREDREQ.genpar,
				 CELL->s_parcres.s_parcel.genpar,
				 sizeof(s_CHUTEREDREQ.genpar));


	SEND_MES(CHUTEREDREQ, MS_NONE, s_id_token_mbx_srldmes.id);

	SETBIT(CELL->status,CELL_WAIT_REDEST);
}

/*@..........................................................................
@  end of the send_redestination_message routine
@***************************************************************************/

/*@**************************************************************************
@
@ FUNCTION      :   compute_average
@
@ DESCRIPTION   :   compute the velocity media in the last n_sample cell
@
@ PROTOTYPE     :   USHORT compute_average(USHORT n_sample)
@
@ PARAMETERS    :   n_sample = number of sample on which the velocity media
@                             must be calculated
@
@ RETURN        :   the velocity average
@
@ NOTES         :
@
***************************************************************************/
USHORT compute_average(USHORT n_sample)
{
long vel,sum_time,value;
int i,loc_idx_sample;
long max_value=0;
long min_value=0xffff;

	if (XBVEL->s_wk_vel.sample_full)
		{
		for (i=n_sample,loc_idx_sample=XBVEL->s_wk_vel.idx_sample,sum_time=0;
				 i;
				 i--)
			{
			if (--loc_idx_sample<0)
				loc_idx_sample+=MAX_VEL_SAMPLE;
			value=XBVEL->s_wk_vel.sample_value[loc_idx_sample];
			if (min_value>value)
				min_value=value;
			if (max_value<value)
				max_value=value;
			sum_time+=value;
			}
		}
	 else if (!XBVEL->s_wk_vel.idx_sample)
			return(0);

	 else
		{
		for (i=n_sample,loc_idx_sample=XBVEL->s_wk_vel.idx_sample,sum_time=0;
				 loc_idx_sample && i;
				 i--)
			{
			value=XBVEL->s_wk_vel.sample_value[--loc_idx_sample];
			if (min_value>value)
				min_value=value;
			if (max_value<value)
				max_value=value;
			sum_time+=value;
			}
		if (XBVEL->s_wk_vel.idx_sample<n_sample)
			n_sample=XBVEL->s_wk_vel.idx_sample;
		}

		if (n_sample>5)
			{
			n_sample-=2;
			sum_time-=(min_value+max_value);
			}
		vel= ((long)XB->s_physchar.cell_pitch * (long)100 * (long)n_sample) /
				 sum_time;

	return(vel);
}

/*@..........................................................................
@  end of compute_average routine
@***************************************************************************/

/*@**************************************************************************
@
@ FUNCTION      : get_offset_ph0
@
@ DESCRIPTION   : from the cell number, by the absolute encoder,
@                 it returns the instantaneous offset_ph0 of the cell
@
@ SYNOPSYS      : USHORT get_offset_ph0(USHORT n_cell);
@
@ PARAMETERS    : n_cell = cell number
@
@ RETURN        : offset from the cell number to the PH0 of encoder
@
@ NOTE          : none
@
@***************************************************************************/
USHORT get_offset_ph0(USHORT n_cell)
{
short offset_ph0;

	if (!SINGLE_DECK_SORTER && n_cell>1000)
		offset_ph0=(XBENC->s_wk_enc.absolute_ep-((n_cell-1001)*XB->s_wk_sorter.n_ep_cell));
	else
		offset_ph0=(XBENC->s_wk_enc.absolute_ep-((n_cell-1)*XB->s_wk_sorter.n_ep_cell));

	/* Mike 30/11/02 Cell pitch is different between odd and even cell*/
	if (TESTBIT(XB->s_logchar.gen_purpose,VARIABLE_PITCH_SORTER) && !(n_cell%2))
		offset_ph0++;

	if (offset_ph0<0)
		offset_ph0+=XB->s_wk_sorter.n_ep_sorter;

	if (!SINGLE_DECK_SORTER && n_cell>1000)
		return (USHORT)offset_ph0+XB->s_wk_sorter.n_ep_sorter;
	else
		return (USHORT)offset_ph0;
}
/*@..........................................................................
@  end of the get_offset_ph0 routine
@***************************************************************************/


